import { render, screen } from '@testing-library/react';
import userEvent from '@testing-library/user-event';
import { useForm } from 'react-hook-form';

import { RoleSelect } from './RoleSelect';

interface TestForm {
  testRole: string;
}

describe('RoleSelect', () => {
  beforeEach(() => {
    jest.spyOn(console, 'error').mockImplementation(() => {});
  });

  afterEach(() => {
    jest.restoreAllMocks();
  });

  const setup = (initialValue = '[]') => {
    const user = userEvent.setup();
    const TestComponent = () => {
      const { control } = useForm<TestForm>({
        defaultValues: {
          testRole: initialValue
        }
      });

      return (
        <RoleSelect<TestForm>
          name="testRole"
          label="Test Role"
          description="Test description"
          control={control}
          inputId="role-select"
        />
      );
    };

    return {
      user,
      ...render(<TestComponent />)
    };
  };

  it('should render with default values', () => {
    setup();
    expect(screen.getByText('Test Role')).toBeInTheDocument();
    expect(screen.getByText('Test description')).toBeInTheDocument();
    expect(screen.getByRole('combobox')).toBeInTheDocument();
  });

  it('should add new value when typing and pressing enter', async () => {
    const { user } = setup();
    const input = screen.getByRole('combobox');

    await user.click(input);
    await user.type(input, 'new-role');
    await user.keyboard('{Enter}');

    expect(screen.getByText('new-role')).toBeInTheDocument();
  });

  it('should handle invalid JSON gracefully', async () => {
    const consoleSpy = jest.spyOn(console, 'error');
    setup('[invalid-json');

    expect(screen.getByRole('combobox')).toBeInTheDocument();
    expect(consoleSpy).toHaveBeenCalledWith('Failed to parse JSON value:', '[invalid-json');
  });

  it('should handle plain string format gracefully', () => {
    setup('admin');

    expect(screen.getByRole('combobox')).toBeInTheDocument();
    expect(screen.getByText('admin')).toBeInTheDocument();

    const consoleSpy = jest.spyOn(console, 'error');
    expect(consoleSpy).not.toHaveBeenCalled();
  });

  it('should handle comma-separated values', () => {
    setup('admin,editor,viewer');

    expect(screen.getByRole('combobox')).toBeInTheDocument();
    expect(screen.getByText('admin')).toBeInTheDocument();
    expect(screen.getByText('editor')).toBeInTheDocument();
    expect(screen.getByText('viewer')).toBeInTheDocument();

    const consoleSpy = jest.spyOn(console, 'error');
    expect(consoleSpy).not.toHaveBeenCalled();
  });

  it('should handle comma-separated values with spaces', () => {
    setup('admin, editor, viewer');

    expect(screen.getByRole('combobox')).toBeInTheDocument();
    expect(screen.getByText('admin')).toBeInTheDocument();
    expect(screen.getByText('editor')).toBeInTheDocument();
    expect(screen.getByText('viewer')).toBeInTheDocument();

    const consoleSpy = jest.spyOn(console, 'error');
    expect(consoleSpy).not.toHaveBeenCalled();
  });

  it('should handle space-separated values', () => {
    setup('admin editor viewer');

    expect(screen.getByRole('combobox')).toBeInTheDocument();
    expect(screen.getByText('admin')).toBeInTheDocument();
    expect(screen.getByText('editor')).toBeInTheDocument();
    expect(screen.getByText('viewer')).toBeInTheDocument();

    const consoleSpy = jest.spyOn(console, 'error');
    expect(consoleSpy).not.toHaveBeenCalled();
  });

  it('should always output JSON array format when adding new values', async () => {
    const TestComponent = () => {
      const { control, watch } = useForm<TestForm>({
        defaultValues: {
          testRole: 'admin'  // Start with plain string
        }
      });

      const value = watch('testRole');

      return (
        <>
          <RoleSelect<TestForm>
            name="testRole"
            label="Test Role"
            description="Test description"
            control={control}
            inputId="role-select"
          />
          <div data-testid="current-value">{value}</div>
        </>
      );
    };

    const user = userEvent.setup();
    render(<TestComponent />);
    const input = screen.getByRole('combobox', { name: 'Test Role' });

    // Verify initial value is displayed
    expect(screen.getByLabelText(/remove/i)).toBeInTheDocument();

    // Add a new value
    await user.click(input);
    await user.type(input, 'editor');
    await user.keyboard('{Enter}');

    // Verify both values are present and format is JSON array
    const currentValue = screen.getByTestId('current-value').textContent;
    expect(currentValue).toBe('["admin","editor"]');
  });

  it('should handle removing values correctly', async () => {
    const TestComponent = () => {
      const { control, watch } = useForm<TestForm>({
        defaultValues: {
          testRole: 'admin,editor,viewer'
        }
      });

      const value = watch('testRole');

      return (
        <>
          <RoleSelect<TestForm>
            name="testRole"
            label="Test Role"
            description="Test description"
            control={control}
            inputId="role-select"
          />
          <div data-testid="current-value">{value}</div>
        </>
      );
    };

    const user = userEvent.setup();
    render(<TestComponent />);

    // Verify initial values
    expect(screen.getByText('admin')).toBeInTheDocument();
    expect(screen.getByText('editor')).toBeInTheDocument();
    expect(screen.getByText('viewer')).toBeInTheDocument();

    // Remove 'editor' value
    const removeButtons = screen.getAllByLabelText(/remove/i);
    await user.click(removeButtons[1]); // Click the second remove button (for 'editor')

    // Verify 'editor' is removed and format is JSON array
    const currentValue = screen.getByTestId('current-value').textContent;
    expect(currentValue).toBe('["admin","viewer"]');
    expect(screen.queryByText('editor')).not.toBeInTheDocument();
  });

  it('should trim whitespace from input values', async () => {
    const { user } = setup();
    const input = screen.getByRole('combobox');

    await user.click(input);
    await user.type(input, '  spaced-role  ');
    await user.keyboard('{Enter}');

    expect(screen.getByText('spaced-role')).toBeInTheDocument();
  });
});
