import { Control, Controller, FieldValues, Path } from 'react-hook-form';

import { Field, Select } from '@grafana/ui';

interface RoleSelectProps<T extends FieldValues> {
  name: Path<T>;
  label: string;
  description: string;
  control: Control<T>;
  inputId?: string;
}

const parseSelectValues = (value: string) => {
  if (!value) {
    return [];
  }
  // Handle all three formats:
  // 1. Single value: "admin"
  // 2. Comma-separated: "admin,editor"
  // 3. Space-separated: "admin editor"
  return value
    .split(/[,\s]+/) // Split on comma or whitespace
    .map((v: string) => v.trim())
    .filter(Boolean) // Remove empty strings
    .map((v: string) => ({label: v, value: v}));
};

const parseValue = (value: string | undefined) => {
  if (!value) {
    return [];
  }
  
  // If it's already a JSON array string, parse it
  if (value.startsWith('[')) {
    try {
      const parsedValue = JSON.parse(value);
      return Array.isArray(parsedValue) ?
        parsedValue.map(v => ({label: v, value: v})) :
        [];
    } catch (e) {
      console.error('Failed to parse JSON value:', value);
      return parseSelectValues(value);
    }
  }
  
  // Handle all other formats using parseSelectValues
  return parseSelectValues(value);
};

/**
 * RoleSelect is a reusable component that provides a multi-select input for role values.
 * It handles multiple input formats:
 * 1. Single string value: "admin"
 * 2. Comma-separated strings: "admin,editor"
 * 3. Space-separated strings: "admin editor"
 * 4. JSON array strings: ["admin","editor"]
 *
 * The component will always output a JSON array string when saving.
 *
 * @example
 * <RoleSelect<FormType>
 *   name="roleValues"
 *   label="Role"
 *   description="Select role values"
 *   control={control}
 *   inputId="role-select"
 * />
 */
export const RoleSelect = <T extends FieldValues>({ name, label, description, control, inputId }: RoleSelectProps<T>) => (
  <Controller<T>
    name={name}
    control={control}
    render={({field: {onChange, value}}) => (
      <Field label={label} description={description}>
        <Select
          inputId={inputId}
          aria-label={label}
          value={parseValue(value)}
          width={60}
          onChange={(options) => {            
            const values = options ? options.map((opt: { value: string }) => opt.value) : [];
            onChange(JSON.stringify(values));
          }}
          options={[]}
          isMulti={true}
          allowCustomValue={true}
          placeholder="Enter role values and press Enter to add"
          onCreateOption={(input: string) => {
            const trimmedInput = input.trim();
            if (!trimmedInput) {
              return;
            }

            const currentValues = value ? 
              (value.startsWith('[') ? 
                JSON.parse(value || '[]') : 
                parseSelectValues(value).map(opt => opt.value)
              ) : 
              [];

            onChange(JSON.stringify([...currentValues, trimmedInput]));
          }}
        />
      </Field>
    )}
  />
);
